@@ -1,6 +1,16 @@
 # coding:utf-8
 from tools_hjh import other as tools
-import time
+from tools_hjh.ThreadPool import ThreadPool
+
+
+def main():
+    testSql = 'insert into t1 select ? where not exists(select 1 from t1 where c1 = ?)'
+    testDB = DBConn('sqlite', db='test.db')
+    tp = ThreadPool(64)
+    for idx in range(0, 9999):
+        tp.run(testDB.run, (testSql, (idx, idx)))
+    tp.wait()
+    testDB.close()
 
 
 class DBConn:
@@ -10,9 +20,10 @@ class DBConn:
         """ 初始化连接池
                 如果是sqlite，db这个参数是要显示给入的
                 如果是oracle，db给入的是sid或是servername都是可以的 """
+        self.runTp = ThreadPool(1)
         self.dbpool = None
         self.dbtype = dbtype
-        config = {
+        self.config = {
             'host':host,
             'port':port,
             'database':db,
@@ -24,12 +35,13 @@ class DBConn:
         }
         if self.dbtype == 'pgsql' or self.dbtype == 'mysql':
             from dbutils.pooled_db import PooledDB
+            
         if self.dbtype == "pgsql":
             import psycopg2
-            self.dbpool = PooledDB(psycopg2, **config)
+            self.dbpool = PooledDB(psycopg2, **self.config)
         elif self.dbtype == "mysql":
             import pymysql
-            self.dbpool = PooledDB(pymysql, **config)
+            self.dbpool = PooledDB(pymysql, **self.config)
         elif self.dbtype == "sqlite": 
             import sqlite3
             from dbutils.persistent_db import PersistentDB
@@ -55,14 +67,7 @@ class DBConn:
                                                     increment=1,
                                                     encoding="UTF-8")
     
-    def run(self, sql, param=None):
-        """ 执行点什么
-        sql中的占位符可以统一使用“?”
-                如果param是一个字符串且作为文件路径判断存在则执行此文件
-                如果param是list作为executemany第二个参数执行
-                如果是一个元组作为execute第二个参数执行
-                如果sql是select开头的，则返回col, rows，rows包含全部结果
-        col表示列头组成的一个元组，rows是一个列表，由多个元组组成，每个元组是一行记录 """
+    def __run(self, sql, param=None):
         # 替换占位符
         if self.dbtype == 'pgsql' or self.dbtype == 'mysql':
             sql = sql.replace('?', '%s')
@@ -79,60 +84,63 @@ class DBConn:
             
         cur = conn.cursor()
         
-        try:
-            '''
-            # 执行SQL文件
-            if type(param) == str and os.path.exists(param):
-                file = open(param)
-                sqlStr = file.read()
-                file.close()
-                if self.dbtype == 'pgsql':
-                    cur.execute(sqlStr)
-                elif self.dbtype == 'sqlite':
-                    cur.executescript(sqlStr)
-                elif self.dbtype == 'mysql' or self.dbtype == 'oracle':
-                    pass  # 暂不支持
-                conn.commit()
-                returnMess = self
-            '''
-            
-            # 执行非SELECT语句
-            if not sql.lower().strip().startswith("select"):
-                sql = sql.strip()
-                if type(param) == list:
-                    cur.executemany(sql, param)
-                elif type(param) == tuple:
-                    cur.execute(sql, param)
-                elif param is None:
-                    cur.execute(sql)
-                conn.commit()
-                rownum = cur.rowcount
-                return rownum
-        
-            # 执行SELECT语句
-            if sql.lower().strip().startswith("select"):
-                sql = sql.strip()
-                col = []
-                if param is None:
-                    cur.execute(sql)
-                elif type(param) == tuple:
-                    cur.execute(sql, param)
-                for c in cur.description:
-                    col.append(c[0])
-                rows = cur.fetchall()
-                return rs(tuple(col), rows)
-            
-        except Exception as e:
-            if self.dbtype == 'sqlite' and 'database is locked' in str(e):
-                cur.close()
-                conn.close()
-                time.sleep(1)
-                self.run(sql, param)
-            else:
-                raise e
-        finally:
+        '''
+        # 执行SQL文件
+        if type(param) == str and os.path.exists(param):
+            file = open(param)
+            sqlStr = file.read()
+            file.close()
+            if self.dbtype == 'pgsql':
+                cur.execute(sqlStr)
+            elif self.dbtype == 'sqlite':
+                cur.executescript(sqlStr)
+            elif self.dbtype == 'mysql' or self.dbtype == 'oracle':
+                pass  # 暂不支持
+            conn.commit()
+            returnMess = self
+        '''
+        # 执行非SELECT语句
+        if not sql.lower().strip().startswith("select"):
+            sql = sql.strip()
+            if type(param) == list:
+                cur.executemany(sql, param)
+            elif type(param) == tuple:
+                cur.execute(sql, param)
+            elif param is None:
+                cur.execute(sql)
+            conn.commit()
+            rownum = cur.rowcount
             cur.close()
             conn.close()
+            return rownum
+    
+        # 执行SELECT语句
+        if sql.lower().strip().startswith("select"):
+            sql = sql.strip()
+            col = []
+            if param is None:
+                cur.execute(sql)
+            elif type(param) == tuple:
+                cur.execute(sql, param)
+            for c in cur.description:
+                col.append(c[0])
+            rows = cur.fetchall()
+            cur.close()
+            conn.close()
+            return rs(tuple(col), rows)
+        
+    def run(self, sql, param=None, wait=None):
+        """ 执行点什么
+        sql中的占位符可以统一使用“?”
+        wait为True则会等待当前正在执行的sql，有bug，暂不处理，自用规避"""
+        if wait is None and self.dbtype == 'sqlite' and not sql.lower().strip().startswith("select"):
+            wait = True
+            
+        if wait == True:
+            self.runTp.run(self.__run, (sql, param))
+            self.runTp.wait()
+        else:
+            return self.__run(sql, param)
         
     def insert(self, table, rows):
         """ 往指定table中插入数据，rows是一个多个元组的列表，每个元组表示一组参数；或者是一个元组 """
@@ -459,10 +467,17 @@ class MemoryDB:
       
 
 class rs:
+
     def __init__(self, cols, rows):
         self.cols = cols
         self.rows = rows
+
     def getCols(self):
         return self.cols
+
     def getRows(self):
         return self.rows
+
+
+if __name__ == '__main__':
+    main()
